//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#include "ez_ilda.hpp"

//############################################################################
void tranz(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                i;
    float              amplitude = 0,
                       phase = half_pi;
    out += frame;
    for(i = 0; i < 60; i++)
    {
        frame.ripple(3, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude += (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(3, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(3, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude -= (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(0, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude += (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(0, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);

    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(0, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude -= (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(1, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude += (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(1, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(1, amplitude, 2, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude -= (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(2, amplitude, 4, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude += (.66 / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(2, amplitude, 4, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
    }
    for(i = 0; i < 60; i++)
    {
        frame.ripple(2, amplitude, 4, phase);
        out += frame;
        frame.rotate_colors();
        phase += (two_pi / 61.0);
        amplitude -= (.66 / 61.0);
    }
    out.save_as_ilda("tranz.ild");
    return;
}

//############################################################################
void stars(ez_ilda_frame* p_frame) // individual segments must have no size
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                frame_count,
                       segment_index;
    ez_3D_short        d,
                       center;
    ez_3D_float        s(1.1, 1.1, 1.1);

    out += frame;
    for(frame_count = 0; frame_count < 199; frame_count++)
    {
        for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
        {
            if(frame.scale_segment_around_origin(segment_index, s)) // out of bounds
            {
                center = frame.find_segment_center(segment_index);
                while(!(d.x = rand() % 5000 - 2500)); //  -2500 ~ +2500, not zero
                while(!(d.y = rand() % 5000 - 2500));
                while(!(d.z = rand() % 5000 - 2500));
                d -= center;
                frame.move_segment(segment_index, d); // somewhere near the origin
            }
        }
        out += frame;
    }
    out.save_as_ilda("stars.ild");
}

//############################################################################
void shrink(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                frame_count;
    ez_3D_float        s(0.96, 0.96, 0.96);
    out += frame;
    for(frame_count = 0; frame_count < 149; frame_count++)
    {        
        frame.scale_around_origin(s);
        out += frame;
    }
    out.save_as_ilda("shrink.ild");
}

//############################################################################
void around(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                frame_count,
                       segment_index,
                       total_segments = frame.number_of_segments();
    ez_3D_float        a(0, 0, pi/20);

    out += frame;
    for(segment_index = 0; segment_index < total_segments; segment_index++)
    {
        for(frame_count = 0; frame_count < 10; frame_count++)
        {
            frame.rotate_segment(segment_index, a);
            out += frame;
        }
    }
    for(segment_index = 0; segment_index < total_segments; segment_index++)
    {
        for(frame_count = 0; frame_count < 10; frame_count++)
        {
            frame.rotate_segment(segment_index, a);
            out += frame;
        }
    }
    for(frame_count = 0; frame_count < 20; frame_count++)
    {
        for(segment_index = 0; segment_index < total_segments; segment_index++)
            frame.rotate_segment(segment_index, a);
        out += frame;
    }
    for(frame_count = 0; frame_count < 20; frame_count++)
    {
        for(segment_index = 0; segment_index < total_segments; segment_index++)
            frame.rotate_segment(segment_index, a);
        out += frame;
    }
    for(frame_count = 0; frame_count < 20; frame_count++)
    {
        for(segment_index = 0; segment_index < total_segments; segment_index++)
            frame.rotate_segment(segment_index, a);
        out += frame;
    }
    out.save_as_ilda("around.ild");
    return;
}

//############################################################################
void rotate(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    ez_3D_float        a;
    int                frame_count;
    
    a.x = -pi/100;
    a.y = 0;
    a.z = 0;
    for(frame_count = 0; frame_count < 200; frame_count++)
    {
        frame.rotate_colors();
        out += frame;
        frame.rotate_colors();
        frame.rotate_around_origin(a);
    }

    a.x = 0;
    a.y = pi/100;
    a.z = 0;
    for(frame_count = 0; frame_count < 200; frame_count++)
    {
        frame.rotate_colors();
        out += frame;
        frame.rotate_around_origin(a);
    }

    a.x = 0;
    a.y = 0;
    a.z = pi/100;
    for(frame_count = 0; frame_count < 200; frame_count++)
    {
        frame.rotate_colors();
        out += frame;
        frame.rotate_around_origin(a);
    }

    a.x = pi/100;
    a.y = pi/100;
    a.z = pi/100;
    for(frame_count = 0; frame_count < 200; frame_count++)
    {
        frame.rotate_colors();
        out += frame;
        frame.rotate_around_origin(a);
    }
    out.save_as_ilda("rotate.ild");
    return;
}

//############################################################################
void bounce(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                frame_count,
                       out_of_bounds;
    ez_3D_float        r;
    ez_3D_short        d;

    d.x = short(ez_random_neg_to_pos_1() * 3000);
    d.y = short(ez_random_neg_to_pos_1() * 3000);
    d.z = short(ez_random_neg_to_pos_1() * 3000);
    r.x = ez_random_neg_to_pos_1() * pi/40.0;
    r.y = ez_random_neg_to_pos_1() * pi/40.0;
    r.z = ez_random_neg_to_pos_1() * pi/40.0;

    out += frame;
    for(frame_count = 0; frame_count < 499; frame_count++)
    {
        frame.rotate(r);                        
        while((out_of_bounds = frame.move(d)))
        {
            switch(out_of_bounds)
            {
                //--------------------------------------------
                case EZ_OUT_POS_X:
                case EZ_OUT_NEG_X:    d.x = -d.x;
                                      r.x = -r.x;
                                      break;
                //--------------------------------------------
                case EZ_OUT_POS_Y:
                case EZ_OUT_NEG_Y:    d.y = -d.y;
                                      r.y = -r.y;
                                      break;
                //--------------------------------------------
                case EZ_OUT_POS_Z:
                case EZ_OUT_NEG_Z:    d.z = -d.z;
                                      r.z = -r.z;
                                      break;
                //--------------------------------------------
            }
        }
        out += frame;
        frame.rotate_colors();
    }
    out.save_as_ilda("bounce.ild");
    return;
}

//############################################################################
void spin(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame(*p_frame);
    int                frame_count,
                       segment_index,
                       out_of_bounds,
                       total_segments = frame.number_of_segments();
    ez_3D_float*       r = new ez_3D_float[total_segments];
    ez_3D_short*       d = new ez_3D_short[total_segments];

    for(segment_index = 0; segment_index < total_segments; segment_index++)
    {
        d[segment_index].x = short(ez_random_neg_to_pos_1() * 1000);
        d[segment_index].y = short(ez_random_neg_to_pos_1() * 1000);
        d[segment_index].z = short(ez_random_neg_to_pos_1() * 500);
        r[segment_index].x = ez_random_neg_to_pos_1() * pi/20.0;
        r[segment_index].y = ez_random_neg_to_pos_1() * pi/20.0;
        r[segment_index].z = ez_random_neg_to_pos_1() * pi/20.0;
    }

    out += frame;
    for(frame_count = 0; frame_count < 299; frame_count++)
    {
        for(segment_index = 0; segment_index < total_segments; segment_index++)
        {                        
            frame.rotate_segment(segment_index, r[segment_index]);                        
            while((out_of_bounds = frame.move_segment(segment_index, d[segment_index])))
            {
                switch(out_of_bounds)
                {
                    case EZ_OUT_POS_X:
                    case EZ_OUT_NEG_X:    d[segment_index].x = -d[segment_index].x;
                                          r[segment_index].x = -r[segment_index].x;
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Y:
                    case EZ_OUT_NEG_Y:    d[segment_index].y = -d[segment_index].y;
                                          r[segment_index].y = -r[segment_index].y;
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Z:
                    case EZ_OUT_NEG_Z:    d[segment_index].z = -d[segment_index].z;
                                          r[segment_index].z = -r[segment_index].z;
                                          break;
                    //--------------------------------------------
                }
            }
        }
        out += frame;
        frame.rotate_colors();
    }
    delete d;
    delete r;
    out.save_as_ilda("spin.ild");
    return;
}

//############################################################################
void _break(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    ez_ilda_frame       frame(*p_frame);
    int                 frame_count,
                        segment_index,
                        out_of_bounds;
    ez_3D_short*        d = new ez_3D_short[frame.number_of_segments()];

    for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
    {
        d[segment_index].x = short((ez_random_01() - 0.5) * 3000);
        d[segment_index].y = short((ez_random_01() - 0.5) * 3000);
        d[segment_index].z = short((ez_random_01() - 0.5) * 3000);
    }
    out += frame;
    for(frame_count = 1; frame_count < 299; frame_count++)
    {
        for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
            while((out_of_bounds = frame.move_segment(segment_index, d[segment_index])))
            {
                switch(out_of_bounds)
                {
                    case EZ_OUT_POS_X:
                    case EZ_OUT_NEG_X:    d[segment_index].x = -(d[segment_index].x);
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Y:    
                    case EZ_OUT_NEG_Y:    d[segment_index].y = -(d[segment_index].y);
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Z:
                    case EZ_OUT_NEG_Z:    d[segment_index].z = -(d[segment_index].z);
                                          break;
                    //--------------------------------------------
                }
            }
        out += frame;
        frame.rotate_colors();
    }
    delete d;
    out.save_as_ilda("break.ild");
    return;
}

//############################################################################
void rainbow(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    int                 i, 
                        span =   ez_ilda_pointer->current_palette().last
                               - ez_ilda_pointer->current_palette().first;
    //------------------------------------------------------------------------
    out += *p_frame;
    for(i = 1; i <= span; i++)
    {
        out[i - 1].rotate_colors();
        out += out[i - 1];
    }
    out.save_as_ilda("rainbow.ild");
    return;
}

//############################################################################
void draw(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    ez_ilda_frame       frame;
    int                 i;
    frame += p_frame->at(0);
    frame += p_frame->at(0);
    out += frame;
    //------------------------------------------------------------------------
    for(i = 1; i < p_frame->number_of_points(); i++)
    {
        frame += p_frame->at(i);
        out += frame;
    }
    out.save_as_ilda("draw.ild");
    return;
}

//############################################################################
void wobble(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    int                 frame_count,
                        segment_index,
                        out_of_bounds;
    ez_ilda_frame       frame(*p_frame);
    ez_3D_float*        a = new ez_3D_float[frame.number_of_segments()];

    for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
    {
        a[segment_index].x = (ez_random_01() - 0.5) * pi/20;
        a[segment_index].y = (ez_random_01() - 0.5) * pi/20;
        a[segment_index].z = (ez_random_01() - 0.5) * pi/20;
    }

    out += frame;
    for(frame_count = 1; frame_count < 300; frame_count++)
    {
        for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
            while((out_of_bounds = frame.rotate_segment(segment_index, a[segment_index])))
            {
                switch(out_of_bounds)
                {
                    case EZ_OUT_POS_X:
                    case EZ_OUT_NEG_X:    a[segment_index].x = -a[segment_index].x;
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Y:
                    case EZ_OUT_NEG_Y:    a[segment_index].y = -a[segment_index].y;
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Z:
                    case EZ_OUT_NEG_Z:    a[segment_index].z = -a[segment_index].z;
                                          break;
                    //--------------------------------------------
                }
            }
        out += frame;
        frame.rotate_colors();
    }
    delete a;
    out.save_as_ilda("wobble.ild");
}

//############################################################################
void wiggle(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    ez_ilda_frame       frame(*p_frame);
    int                 frame_count,
                        segment_index;
    ez_3D_float         d;
                        
    out += frame;

    for(frame_count = 1; frame_count < 199; frame_count++)
    {
        for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
        {
            do
            {
                d.x = (ez_random_01() - 0.5) * 1000;
                d.y = (ez_random_01() - 0.5) * 1000;
                d.z = (ez_random_01() - 0.5) * 1000;
            } while(frame.move_segment(segment_index, d));
        }
        out += frame;
    }
    out.save_as_ilda("wiggle.ild");
    return;
}

//############################################################################
void scan(ez_ilda_frame* p_frame)
{
    ez_ilda_frame_set   out;
    ez_ilda_frame       frame(*p_frame);
    int                 frame_count,
                        segment_index,
                        out_of_bounds;
    ez_3D_short         d(1200,  666,     0);
    ez_3D_float         a(   0,    0, pi/30);

    out += frame;
    for(frame_count = 0; frame_count < 299; frame_count++)
    {
        for(segment_index = 0; segment_index < frame.number_of_segments(); segment_index++)
        {
            frame.rotate_segment(segment_index, a);
            while((out_of_bounds = frame.move_segment(segment_index, d)))
            {
                switch(out_of_bounds)
                {
                    //--------------------------------------------
                    case EZ_OUT_POS_X:
                    case EZ_OUT_NEG_X:    d.x = -d.x;
                                          break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Y:
                    case EZ_OUT_NEG_Y:    d.y = -d.y;
                                          break;
                    //--------------------------------------------
                }
            }
        }
        out += frame;
    }
    out.save_as_ilda("scan.ild");
    return;
}

//############################################################################
void fall(ez_ilda_frame* p_frame)
{
    ez_ilda_frame       frame(*p_frame);
    ez_ilda_frame_set   segments = frame.find_segments(),
                        out;
    int                 frame_count,
                        segment_index,
                        out_of_bounds;
    ez_3D_float         d;
                        
    out += frame;

    for(frame_count = 1; frame_count < 399; frame_count++)
    {
        for(segment_index = 0; segment_index < segments.number_of_frames(); segment_index++)
        {
            d.x = (ez_random_01() - 0.5) * 1000; // -500 to +500
            d.y = (ez_random_01() - 1  ) * 1000; // always negative. DOWN!
            d.z = (ez_random_01() - 0.5) * 1000; // -500 to +500
            out_of_bounds = segments[segment_index].move(d);
            if(out_of_bounds)
            {
                switch(out_of_bounds)
                {
                    case EZ_OUT_POS_X:
                    case EZ_OUT_NEG_X:  d.x = -d.x;
                                        break;
                    //--------------------------------------------
                    case EZ_OUT_NEG_Y:  d.y = EZ_MAX_SHORT - segments[segment_index].segment_top().y;
//                    case EZ_OUT_NEG_Y:  d.y = 0;
                                        break;
                    //--------------------------------------------
                    case EZ_OUT_POS_Z:
                    case EZ_OUT_NEG_Z:  d.z = -d.z;
                                        break;
                    //--------------------------------------------
                }
                out_of_bounds = segments[segment_index].move(d);
            }
        }
        frame = segments.sum_of_frames();
        out += frame;
    }
    out.save_as_ilda("fall.ild");
    return;
}

//############################################################################






//############################################################################
void ez_ilda::load_frame_effects()
{
    frame_effects.push_back(tranz);
    frame_effect_names.push_back(string("tranz"));
    //------------------------------------------------------------------------
    frame_effects.push_back(stars);
    frame_effect_names.push_back(string("stars"));
    //------------------------------------------------------------------------
    frame_effects.push_back(shrink);
    frame_effect_names.push_back(string("shrink"));
    //------------------------------------------------------------------------
    frame_effects.push_back(around);
    frame_effect_names.push_back(string("around"));
    //------------------------------------------------------------------------
    frame_effects.push_back(rotate);
    frame_effect_names.push_back(string("rotate"));
    //------------------------------------------------------------------------
    frame_effects.push_back(bounce);
    frame_effect_names.push_back(string("bounce"));
    //------------------------------------------------------------------------
    frame_effects.push_back(spin);
    frame_effect_names.push_back(string("spin"));
    //------------------------------------------------------------------------
    frame_effects.push_back(_break);
    frame_effect_names.push_back(string("break"));
    //------------------------------------------------------------------------
    frame_effects.push_back(rainbow);
    frame_effect_names.push_back(string("rainbow"));
    //------------------------------------------------------------------------
    frame_effects.push_back(draw);
    frame_effect_names.push_back(string("draw"));
    //------------------------------------------------------------------------
    frame_effects.push_back(wobble);
    frame_effect_names.push_back(string("wobble"));
    //------------------------------------------------------------------------
    frame_effects.push_back(wiggle);
    frame_effect_names.push_back(string("wiggle"));
    //------------------------------------------------------------------------
    frame_effects.push_back(scan);
    frame_effect_names.push_back(string("scan"));
    //------------------------------------------------------------------------
    frame_effects.push_back(fall);
    frame_effect_names.push_back(string("fall"));
    //------------------------------------------------------------------------
    return;
}

//############################################################################
